#This file is part of numword.  The COPYRIGHT file at the top level of
#this repository contains the full copyright notices and license terms.
'''
numword for EU
'''

from .numword_base import NumWordBase


class NumWordEU(NumWordBase):
    '''
    NumWord EU
    '''

    def _set_high_numwords(self, high):
        '''
        Set high num words
        '''
        max_val = 3 + 6 * len(high)

        for word, i in zip(high, range(max_val, 3, -6)):
            self.cards[10 ** i] = word + u"illiard"
            self.cards[10 ** (i - 3)] = word + u"illion"

    def _base_setup(self):
        '''
        Base setup
        '''
        lows = [
            u"non",
            u"oct",
            u"sept",
            u"sext",
            u"quint",
            u"quadr",
            u"tr",
            u"b",
            u"m",
            ]
        units = [
            u"",
            u"un",
            u"duo",
            u"tre",
            u"quattuor",
            u"quin",
            u"sex",
            u"sept",
            u"octo",
            u"novem",
            ]
        tens = [
            u"dec",
            u"vigint",
            u"trigint",
            u"quadragint",
            u"quinquagint",
            u"sexagint",
            u"septuagint",
            u"octogint",
            u"nonagint",
            ]
        self.high_numwords = ([u"cent"]
            + self._gen_high_numwords(units, tens, lows))

    def currency(self, value, longval=True, jointxt=u"", hightxt=u"Euro/s",
            lowtxt=u"Euro cent/s", space=True):
        '''
        Convert to currency
        '''
        return self._split(value, hightxt=hightxt, lowtxt=lowtxt,
            jointxt=jointxt, longval=longval, space=space)

    def _merge(self, curr, next):
        '''
        Merge
        '''
        raise NotImplementedError



# -*- coding: utf-8 -*-
#This file is part of numword.  The COPYRIGHT file at the top level of
#this repository contains the full copyright notices and license terms.
'''
numword for ES
'''

from .numword_eu import NumWordEU

#TODO correct orthographics
#TODO error messages


class NumWordES(NumWordEU):
    '''
    NumWord ES
    '''

    def __init__(self):
        self.gender_stem = None
        super(NumWordES, self).__init__()

    #TODO Is this sufficient??
    def _set_high_numwords(self, high):
        '''
        Set high numwords
        '''
        max_val = 3 + 6 * len(high)

        for word, i in zip(high, range(max_val, 3, -6)):
            self.cards[10 ** (i - 3)] = word + u"illón"

    def _setup(self):
        '''
        Setup
        '''
        lows = [u"cuatr", u"tr", u"b", u"m"]
        self.high_numwords = self._gen_high_numwords([], [], lows)
        self.negword = u"menos "
        self.pointword = u"punto"
        self.gender_stem = u"o"
        self.exclude_title = [u"y", u"menos", u"punto"]
        self.mid_numwords = [
            (1000, u"mil"),
            (100, u"cien"),
            (90, u"noventa"),
            (80, u"ochenta"),
            (70, u"setenta"),
            (60, u"sesenta"),
            (50, u"cincuenta"),
            (40, u"cuarenta"),
            (30, u"treinta"),
            ]
        self.low_numwords = [
            u"veintinueve",
            u"veintiocho",
            u"veintisiete",
            u"veintiséis",
            u"veinticinco",
            u"veinticuatro",
            u"veintitrés",
            u"veintidós",
            u"veintiuno",
            u"veinte",
            u"diecinueve",
            u"dieciocho",
            u"diecisiete",
            u"dieciseis",
            u"quince",
            u"catorce",
            u"trece",
            u"doce",
            u"once",
            u"diez",
            u"nueve",
            u"ocho",
            u"siete",
            u"seis",
            u"cinco",
            u"cuatro",
            u"tres",
            u"dos",
            u"uno",
            u"cero",
            ]
        self.ords = {
            1: u"primer",
            2: u"segund",
            3: u"tercer",
            4: u"cuart",
            5: u"quint",
            6: u"sext",
            7: u"séptim",
            8: u"octav",
            9: u"noven",
            10: u"décim",
            }

    def _merge(self, curr, next):
        '''
        Merge
        '''
        ctext, cnum, ntext, nnum = curr + next

        if cnum == 1:
            if nnum < 1000000:
                return next
            ctext = u"un"
        elif cnum == 100:
            if nnum != 1000:
                ctext += u"t" + self.gender_stem
        elif str(cnum)[-1] == '1':
            if ctext[-1] == u"o":
                ctext = ctext[:-1]
        if nnum < cnum:
            if cnum < 100:
                return (u"%s y %s" % (ctext, ntext), cnum + nnum)
            return (u"%s %s" % (ctext, ntext), cnum + nnum)
        elif (not nnum % 1000000) and cnum > 1:
            ntext = ntext[:-3] + u"lones"

        if nnum == 100:
            if cnum == 5:
                ctext = u"quinien"
                ntext = u""
            elif cnum == 7:
                ctext = u"sete"
            elif cnum == 9:
                ctext = u"nove"
            ntext += u"t" + self.gender_stem + u"s"
        else:
            ntext = u" " + ntext

        return (ctext + ntext, cnum * nnum)

    def ordinal(self, value):
        '''
        Convert to ordinal
        '''
        self._verify_ordinal(value)
        try:
            return self.ords[value] + self.gender_stem
        except KeyError:
            return self.cardinal(value)

    def ordinal_number(self, value):
        '''
        Convert to ordinal number
        '''
        self._verify_ordinal(value)
        # Correct for fem?
        return u"%s°" % value

    def currency(self, val, longval=True, old=False):
        '''
        Convert to currency
        '''
        if old:
            return self._split(val, hightxt=u"peso/s", lowtxt=u"peseta/s",
                divisor=1000, jointxt=u"y", longval=longval)
        return super(NumWordES, self).currency(val, jointxt=u"y",
            longval=longval)

_NW = NumWordES()


def cardinal(value):
    '''
    Convert to cardinal
    '''
    return _NW.cardinal(value)


def ordinal(value):
    '''
    Convert to ordinal
    '''
    return _NW.ordinal(value)


def ordinal_number(value):
    '''
    Convert to ordinal number
    '''
    return _NW.ordinal_number(value)


def currency(value, longval=True, old=False):
    '''
    Convert to currency
    '''
    return _NW.currency(value, longval=longval, old=old)


def year(value, longval=True):
    '''
    Convert to year
    '''
    return _NW.year(value, longval=longval)



# -*- coding: utf-8 -*-
#This file is part of numword.  The COPYRIGHT file at the top level of
#this repository contains the full copyright notices and license terms.
'''
numword for FR
'''

from .numword_eu import NumWordEU


class NumWordFR(NumWordEU):
    '''
    NumWord FR
    '''

    def _setup(self):
        '''
        Setup
        '''
        self.negword = u"moins "
        self.pointword = u"virgule"
        self.exclude_title = [u"et", u"virgule", u"moins"]
        self.mid_numwords = [
            (1000, u"mille"),
            (100, u"cent"),
            (80, u"quatre-vingts"),
            (60, u"soixante"),
            (50, u"cinquante"),
            (40, u"quarante"),
            (30, u"trente")]
        self.low_numwords = [
            u"vingt",
            u"dix-neuf",
            u"dix-huit",
            u"dix-sept",
            u"seize",
            u"quinze",
            u"quatorze",
            u"treize",
            u"douze",
            u"onze",
            u"dix",
            u"neuf",
            u"huit",
            u"sept",
            u"six",
            u"cinq",
            u"quatre",
            u"trois",
            u"deux",
            u"un",
            u"zéro"]

    def _merge(self, curr, next):
        '''
        Merge
        '''
        ctext, cnum, ntext, nnum = curr + next

        if cnum == 1:
            if nnum < 1000000:
                return next
        else:
            if ((not (cnum - 80) % 100 or not cnum % 100)
                    and nnum < 1000000 and ctext[-1] == u"s"):
                ctext = ctext[:-1]
            if (cnum < 1000 and nnum != 1000 and ntext[-1] != "s"
                    and not nnum % 100):
                ntext += "s"

        if (nnum < cnum < 100
                and nnum % 10 == 1 and cnum != 80):
            return (u"%s-et-%s" % (ctext, ntext), cnum + nnum)
        if nnum >= 1000000 or cnum >= 1000000:
            return (u"%s %s" % (ctext, ntext), cnum + nnum)
        return (u"%s-%s" % (ctext, ntext), cnum + nnum)

    def ordinal(self, value):
        '''
        Convert to ordinal
        '''
        self._verify_ordinal(value)
        if value == 1:
            return u"premier"
        word = self.cardinal(value)
        if word[-1] == u"e" or word[-1] == u"s":
            word = word[:-1]
        if word[-1] == u"f":
            word = word[:-1] + u'v'
        return word + u"ième"

    def ordinal_number(self, value):
        '''
        Convert to ordinal number
        '''
        self._verify_ordinal(value)
        out = str(value)
        out += {"1": u"er"}.get(out[-1], u"me")
        return out

    def currency(self, val, longval=True, old=False):
        '''
        Convert to currency
        '''
        hightxt = u"Euro/s"
        if old:
            hightxt = u"franc/s"
        return self._split(val, hightxt=hightxt, lowtxt=u"centime/s",
            jointxt=u"et", longval=longval)

_NW = NumWordFR()


def cardinal(value):
    '''
    Convert to cardinal
    '''
    return _NW.cardinal(value)


def ordinal(value):
    '''
    Convert to ordinal
    '''
    return _NW.ordinal(value)


def ordinal_number(value):
    '''
    Convert to ordinal number
    '''
    return _NW.ordinal_number(value)


def currency(value, longval=True, old=False):
    '''
    Convert to currency
    '''
    return _NW.currency(value, longval=longval, old=old)


def year(value):
    '''
    Convert to year
    '''
    return _NW.year(value)



#This file is part of numword.  The COPYRIGHT file at the top level of
#this repository contains the full copyright notices and license terms.
'''
numword
'''
__version__ = '0.8'

import locale as _locale

_MODULES = []
for _loc in [_locale.getlocale(), _locale.getdefaultlocale()]:
    _lang = _loc[0]
    if _lang:
        _lang = _lang.lower()

        _MODULES.append("numword.numword_" + _lang)
        _MODULES.append("numword.numword_" + _lang.split("_")[0])

for _module in _MODULES:
    try:
        nwmod = __import__(_module)
        components = _module.split('.')
        for comp in components[1:]:
            nwmod = getattr(nwmod, comp)
        break
    except ImportError:
        continue

try:
    def cardinal(value):
        '''
        Convert to cardinal
        '''
        return nwmod.cardinal(value)

    def ordinal(value):
        '''
        Convert to ordinal
        '''
        return nwmod.ordinal(value)

    def ordinal_number(value):
        '''
        Convert to ordinal number
        '''
        return nwmod.ordinal_number(value)

    def currency(value, **kwargs):
        '''
        Convert to currency
        '''
        return nwmod.currency(value, **kwargs)

    def year(value, **kwargs):
        '''
        Convert to year
        '''
        return nwmod.year(value, **kwargs)

except NameError:
    raise ImportError("Could not import any of these modules: %s"
        % (", ".join(_MODULES)))



#This file is part of numword.  The COPYRIGHT file at the top level of
#this repository contains the full copyright notices and license terms.
'''
numword for EN_GB
'''

from .numword_en import NumWordEN


class NumWordENGB(NumWordEN):
    '''
    NumWord EN_GB
    '''

    def currency(self, val, longval=True):
        '''
        Convert to currency
        '''
        return self._split(val, hightxt=u"pound/s", lowtxt=u"pence",
            jointxt=u"and", longval=longval)

_NW = NumWordENGB()


def cardinal(value):
    '''
    Convert to cardinal
    '''
    return _NW.cardinal(value)


def ordinal(value):
    '''
    Convert to ordinal
    '''
    return _NW.ordinal(value)


def ordinal_number(value):
    '''
    Convert to ordinal number
    '''
    return _NW.ordinal_number(value)


def currency(value, longval=True):
    '''
    Convert to currency
    '''
    return _NW.currency(value, longval=longval)


def year(value, longval=True):
    '''
    Convert to year
    '''
    return _NW.year(value, longval=longval)



# -*- coding: utf-8 -*-
#This file is part of numword.  The COPYRIGHT file at the top level of
#this repository contains the full copyright notices and license terms.
'''
numword for FR_BE
'''

from .numword_fr import NumWordFR


class NumWordFRBE(NumWordFR):
    '''
    NumWord FR_BE
    '''

    def _setup(self):
        '''
        Setup
        '''
        super(NumWordFRBE, self)._setup()
        self.mid_numwords.insert(3, (70, u"septante"))
        self.mid_numwords.insert(2, (90, u"nonante"))

_NW = NumWordFRBE()


def cardinal(value):
    '''
    Convert to cardinal
    '''
    return _NW.cardinal(value)


def ordinal(value):
    '''
    Convert to ordinal
    '''
    return _NW.ordinal(value)


def ordinal_number(value):
    '''
    Convert to ordinal number
    '''
    return _NW.ordinal_number(value)
